#!/bin/bash

#######
#
# Script para actualizar G-Radio!!!
#
# USARLO CON MUCHO CUIDADO BAJO SU PROPIA RESPONSABILIDAD!!!
# NO RECOMENDAMOS EL USO EN EQUIPOS QUE ESTEN EN PRODUCCION.
#
# =======
# Este script tiene como función:
# * Descargar desde el sitio oficial de G-Radio SOLO los archivos que han sido modificando
# * Reemplazar los scripts y archivos de configuración de cada versión
# * Asignar los permisos correspondientes
# * El script es independientemente de la distribución
# =======
#
# Este script fue escrito originalmente por Javier Obregón (a.k.a 4rkng3l) - https://libertics.com.ar/
# bajo Licencia GPL 2 o superior para colaborar con el equipo de G-Radio - http://gradio.net/
#######

# Variables
DIR_GR="$HOME/.gradio"
ACTUALIZAR=1	# 0 = Actualiza 1 = NO Actualiza
DEPENDENCIAS=""
DIR_UP="$DIR_GR/.gradio.update"
ARCHLOG="$DIR_GR/data/tmp/gr-update.log"
ARCHUP="listado-actualizar.txt"
FIRMALOCAL="sha256sums-local"
FIRMAREMOTA="sha256sums"

## RELEASE CANDIDATE 1 desarrollo de versión 1.6 con actualización selectiva
URLGRADIO="http://www.g-radio.org/descargas/update-RC1-4.0/"
DIR_DESCARGA="update-RC1-4.0"

## Variables agregadas en la v1.9 del script función descopmrimirwwww
DIRREPWWW="www.tar.gz"  # Nombre del archivo comprimido que contiene el DIRectorio www para el REPorte WWWWeb
echo 1 > /tmp/DESCOMWWW # Semáforo para acceder o no a la función donde descomprimirwww() 0 = Accede 1 = NO Accede

## Establece explícitamente DIR_GR como directorio de trabajo para el script
cd $DIR_GR

if [[ -f $DIR_GR/data/version.txt ]]
then
	OLDVER=$(cat $DIR_GR/data/version.txt | tr -d ' ')
fi

# Saca el idioma del sistema
IDI=$(echo $LANG | cut -d _ -f1)
# Mensajes...
if [[ $IDI = "pt" ]]
then
	# en portugues..
	# Sacy - Rádio Comunitária Aconchego
    MSJ1="Não se pôde gerar o arquivo de hashes, não poderás continuar.\nEquipe G-Radio"
    MSJ2="Não se pôde baixar o arquivo de hashes do servidor de G-Radio, não poderás continuar.\nEquipe G-Radio"
	MSJ4="G-Radio se atualizou com éxito!!!.\nEquipe G-Radio"
	MSJ5="Esta versão de G-Radio requer que tenhas instaladas as seguintes dependências:\n\n-> $DEPENDENCIAS\n\nNão se esqueça de instalar-las antes de executar G-Radio.\nEquipe G-Radio"
	MSJ6="Ocorreu um erro durante o processo de atualização.\nTente novamente mais tarde.\nEquipe G-Radio"
	MSJ7="A atualização de G-Radio foi cancelada.\nEquipe G-Radio"
	MSJ8="Ocorreu um ERRO inesperado.\nTente novamente mais tarde.\nEquipe G-Radio"
    MSJ9="Não há atualizações disponíveis.\nTente mais tarde.\nEquipe G-Radio"
    MSJ10="O log de atualização não foi gravado.\nEquipe G-Radio"
    MSJ11="Não há nenhum arquivo $DIRREPWWW .\nEquipe G-Radio"
elif [[ $IDI = "en" ]]
then	
	# en inglés..
	# thanks Marisol Páez
	MSJ1="The hashes file could not be generated, it cannot be continued.\nG-Radio Team"
	MSJ2="The hashes file could not be downloaded from the G-Radio server, it cannot be continued\nG-Radio Team"
	MSJ3="There seems to be no internet connection, or the G-Radio site is not responding at this time.\nCheck out and try again. \nG-Radio Team"
	MSJ4="G-Radio was successfully updated!!! \nG-Radio Team"
	MSJ5="This version of G-Radio requires the installation of the following dependencies:\n\n-> $DEPENDENCIAS\n\nDon't forget to install that before execute G-Radio.\nG-Radio Team"
	MSJ6="An error has occurred during the update process.\nTry again later.\nG-Radio Team"
	MSJ7="The update of G- radio has been cancelled.\nG-Radio Team"
	MSJ8="An unexpected ERROR has ocurred.\nTry again later.\nG-Radio Team"
    MSJ9="No updates available.\nTry it later.\nG-Radio Team"
    MSJ10="The update log was not saved.\nG-Radio Team"
    MSJ11="The file does not exist $DIRREPWWW .\nG-Radio Team"
else
	# en español (PREDETERMINADO)..
	MSJ1="No se pudo generar el archivo de hashes, no se puede continuar.\nEquipo G-Radio"
	MSJ2="No se pudo descargar el archivo de hashes desde el servidor de G-Radio, no se puede continuar.\nEquipo G-Radio"
	MSJ3="Parece que no hay conexión a internet, o bien el sitio de G-Radio no está respondiendo en éste momento.\nRevisa y vuelve a intentar.\nEquipo G-Radio"
	MSJ4="G-Radio se actualizó con éxito!!!.\nEquipo G-Radio"
	MSJ5="Esta versión de G-Radio requiere que tengas instaladas las siguientes dependencias:\n\n-> $DEPENDENCIAS\n\nNo te olvides de instalar antes de ejecutar G-Radio.\nEquipo G-Radio"
	MSJ6="Se ha producido un error durante el proceso de actualización.\nIntentelo más tarde.\nEquipo G-Radio"
	MSJ7="Se ha cancelado la actualización de G-Radio.\nEquipo G-Radio"
	MSJ8="Se ha producido un ERROR inesperado.\nIntentelo más tarde.\nEquipo G-Radio"
	MSJ9="No hay actualizaciones disponibles.\nIntentelo más tarde.\nEquipo G-Radio"
    MSJ10="No se guardo el log de actualizaciones.\nEquipo G-Radio"
    MSJ11="No existe el archivo $DIRREPWWW .\nEquipo G-Radio"
fi

##############################
## Declaración de funciones ##
##############################
comprobar_red() {
	CR=$(ping gradio.net -c3 | grep -i "0% packet loss" | awk -F "," '{print $2}' | tr -d ' ')
	if [[ ! -z $CR ]]
	then
		return 0
	else
		return 1
	fi
}
backupgr() {
	## Creación del archivo $DIR_GR/respaldos y reubicación de archivos de respaldo previos
	if [[ ! -d $DIR_GR/respaldos ]]; then
		mkdir $DIR_GR/respaldos
		sleep 0.5 #En los discos sólidos hay problemas si no espera?
		mkdir $DIR_GR/respaldos/data
		mkdir $DIR_GR/respaldos/bin
		mv $DIR_GR/bin-respaldo-* $DIR_GR/respaldos/bin/
		mv $DIR_GR/data/comerciales-respaldo-* $DIR_GR/respaldos/data/
		mv $DIR_GR/data/eventos-respaldo-* $DIR_GR/respaldos/data/
		mv $DIR_GR/data/panel-respaldo-* $DIR_GR/respaldos/data/
		mv $DIR_GR/data/eventos-espera-respaldo-* $DIR_GR/respaldos/data/
		mv $DIR_GR/data/parrilla-respaldo-* $DIR_GR/respaldos/data/
	fi
	## Respaldo de los archivos en $DIR_GR/bin
	cp -R $DIR_GR/bin $DIR_GR/respaldos/bin/bin-respaldo-"`date +%d-%m-%Y-%H:%M:%S`"
	## Respaldo de comerciales eventos panel en $DIR_GR/data/ 
	cp -R $DIR_GR/data/comerciales $DIR_GR/respaldos/data/comerciales-respaldo-"`date +%d-%m-%Y-%H:%M:%S`"
	cp -R $DIR_GR/data/eventos $DIR_GR/respaldos/data/eventos-respaldo-"`date +%d-%m-%Y-%H:%M:%S`"
	cp -R $DIR_GR/data/panel $DIR_GR/respaldos/data/panel-respaldo-"`date +%d-%m-%Y-%H:%M:%S`"
	## Agregado en 2.2.2 - Renegado
	cp -R $DIR_GR/data/eventos-espera $DIR_GR/respaldos/data/eventos-espera-respaldo-"`date +%d-%m-%Y-%H:%M:%S`"
	cp -R $DIR_GR/data/parrilla $DIR_GR/respaldos/data/parrilla-respaldo-"`date +%d-%m-%Y-%H:%M:%S`"
}
actualizacion() {
    CANT=$(wc -l $ARCHUP | awk -F" " '{print $1}')
	if [[ -d $DIR_UP ]]
	then
		## Borra el directorio temporal antes de empezar, por si hubo un intento previo que no termino...
        rm -rf $DIR_UP/
	fi
    # Descargar SOLO los archivos detallados en ARCHUP completados en la función previa listado_actualizar
    # En v1.9 se modifica los parámetros de wget
    (while read linea
    do
        wget -nH -np -nc -r -c -U Mozilla -e robots=off -P $DIR_UP/ $URLGRADIO$linea
    done < $ARCHUP) | zenity --progress --title="Por favor espere..." --text " Descargando $CANT actualizaciones... " --percentage=0 --auto-close --pulsate

if [[ $? -eq 0 ]]
then
 (mv $DIR_UP/descargas/$DIR_DESCARGA/* $DIR_UP/ ;\
 [[ -d $DIR_UP/bin/ ]] && cp -r $DIR_UP/bin/* $HOME/.gradio/bin/ ;\
 [[ -d $DIR_UP/data/ ]] && cp -r $DIR_UP/data/* $HOME/.gradio/data/ ;\
 [[ -d "$HOME"/.gradio/doc ]] || mkdir -p $HOME/.gradio/doc ;\
 chmod +x $HOME/.gradio/bin/* -R ;\
 chmod -x $HOME/.gradio/bin/LEAME-ACTUALIZACION ;\
 chmod -x $HOME/.gradio/bin/cargalista4.py-old ;\
 chmod +x $HOME/.gradio/bin/*.gambas ;\
 chmod +x $HOME/.gradio/bin/mascara/*.gambas ;\
 chmod +x $HOME/.gradio/data/panel/*.sh ;\
 [[ -d $DIR_UP/doc/ ]] && cp -r $DIR_UP/doc/* $HOME/.gradio/doc/ ) | zenity --progress --title="Por favor espere..." --text " Actualizando... " --percentage=0 --auto-close --pulsate
	if [[ $? -eq 0 ]]
	then
		# Actualización Ok...
        return 0
	else
		# Actualización con FALLA...
        return 1
	fi
else
	# Actualización con FALLA...
    return 1
fi
}
## Nuevas funciones para actualización selectiva desde v1.6 (Noviembre 2019)
firmas_locales(){
    (find bin/* -type f -print0 | xargs -0 -n 1 sha256sum > $FIRMALOCAL &&\
    find data/* -type f -print0 | xargs -0 -n 1 sha256sum >> $FIRMALOCAL &&\
    find doc/* -type f -print0 | xargs -0 -n 1 sha256sum >> $FIRMALOCAL) | zenity --progress --title="Creando firmas locales..." --text " Por favor espere... " --percentage=0 --auto-close --pulsate
	if [[ $? -eq 0 ]]
	then
		# Se genera correctamente el archivo de hashes...
        return 0
	else
        # NO se genera el archivo de hashes...
        return 1
	fi
}
firmas_remotas(){
    wget -c $URLGRADIO$FIRMAREMOTA
    if [[ $? -eq 0 ]]
    then
    	# Se descargo correctamente el archivo de hashes desde el servidor...
        return 0
    else
        # NO se descargo el archivo de hashes desde el servidor...
        return 1
    fi
}
listado_actualizar(){
    # Recorre FIRMAREMOTA comparando cada hash con los de FIRMALOCAL si no encuentra significa
    # que es o NUEVO o MODIFICADO y por ende debe ser descargado / actualizado para ello se
    # crea un nuevo archivo de nombre listado-actualizar.txt donde se guarda la ruta del archivo
    # en cuestión en el servidor.
    # Todo este proceso se realiza en un directorio temporal declarado en la variable DIR_UP este directorio 
    # es $HOME/.gradio/.gradio.update de forma predeterminada
    ARCHEXCLUIR="firma-de-cambios sha256sums sha256update.sh"
    if [[ -f $ARCHUP ]]
    then
        rm $ARCHUP
    fi
    # Recorremos el archivo FIRMAREMOTA y tomamos el primer campo de cada línea que es el hash
    (while read linea
    do
        HASHARCHIVO=$(echo $linea | awk -F" " '{print $1}')
        RUTAARCHIVO=$(echo $linea | awk -F" " '{print $2}')

        # En la v1.9 se agrega un nuevo control para identificar archivo www.tar.gz si existe se prende el semáforo
        # DESCWWWW pasando del estado 1 inicial al 0 para correr posteriormente la función descomprimirwww()
        NOMBREARCHIVO=$(echo $linea | awk '{print $2}' | awk -F"/" '{print $NF}')

        # ... y lo buscamos en FIRMALOCAL
        BUSCAR=$(cat $FIRMALOCAL | grep -w $HASHARCHIVO)
        A=0
        if [[ -z $BUSCAR ]]
        then
            # Si $BUSCAR está vacío, es decir NO encontró coincidencia significa que es un archivo NUEVO o ACTUALIZADO
            # asi que lo agregamos al listado-actualizar.txt y se pasa al siguiente hash
            # Pero para ser agregado a RUTAARCHIVO NO DEBE estar en el listado de ARCHEXCLUIR
            for EXCLUIR in $ARCHEXCLUIR
            do
                if [[ $RUTAARCHIVO = $EXCLUIR ]]
                then
                    A=1
                fi
            done
            if [[ $A -eq 0 ]]
            then
                if [[ $NOMBREARCHIVO = $DIRREPWWW ]]
                then
                    ## Si el archivo agregado / modificado es puntualmente www.tar.gz, cambia el semáforo a 0
                    echo 0 > /tmp/DESCOMWWW
                fi
                echo $RUTAARCHIVO >> $ARCHUP
            fi
        fi
    done < $FIRMAREMOTA) | zenity --progress --title="Por favor espere" --text " Buscando actualizaciones... " --percentage=0 --auto-close --pulsate
    if [[ -f $ARCHUP ]]
    then
        # Hay actualizaciones pendientes...
        return 0
    else
        # NO Hay actualizaciones pendientes...
        return 1
    fi
}
log_actualizacion(){
    if [[ -f $ARCHUP ]]
    then
        ## Guardando log de actualización
        echo "-------------" >> $ARCHLOG
        (while read linea
        do
            echo "$(date +%d-%m-%Y-%H:%M:%S),$linea" >> $ARCHLOG
        done < $ARCHUP) | zenity --progress --title="Por favor espere..." --text " Guardando log de actualización... " --percentage=0 --auto-close --pulsate
	    if [[ $? -eq 0 ]]
	    then
	    	# Se guarda el log...
            return 0
	    else
            # NO se guarda el log...
            return 1
	    fi
    else            
            return 0
    fi
}
limpieza(){
    sleep 5
    cd $HOME/.gradio/
    if [[ -f $ARCHUP ]]
    then
        rm $ARCHUP
    fi
    if [[ -f $FIRMALOCAL ]]
    then
        rm $FIRMALOCAL
    fi
    if [[ -f $FIRMAREMOTA ]]
    then
        rm $FIRMAREMOTA
    fi
    if [[ -d $DIR_UP ]]
    then
        rm -rf $DIR_UP/
    fi
    find $HOME/.gradio/ -name *=* -delete
    rm /tmp/DESCOMWWW
    return 0
}
descomprimirwww(){
    ## Función agregada en v1.9 del script y rc 0.29 G-Radio
    ## Esta función se encarga de descomprimir el archivo www.tar.gz
    ## el cual contiene la estructura de directorio para reportes web
    cd $HOME/.gradio/data/
    tar xfs $DIRREPWWW
    # Agrega al log la acción de descopresión de nueva versión de www.tar.gz
    echo "-------------" >> $ARCHLOG
    echo "$(date +%d-%m-%Y-%H:%M:%S),Descompresión de ~/.gradio/data/www/" >> $ARCHLOG
    return 0
}
controldescompresionwww(){
    D1="No"
    if [[ $(cat /tmp/DESCOMWWW) -eq 0 ]]
    then
        D1="Si"
            ## Si hay una nueva versión de www.tar.gz... descomprime
            D=0
    elif [[ -f $HOME/.gradio/data/$DIRREPWWW ]] && [[ ! -d $HOME/.gradio/data/www ]]
    then
        ## Si existe www.tar.gz y no existe directorio www/... descomprime
        D=0
    else
        ## Si no hay actualización en www.tar.gz y wwww/ ya existe... NO descomprime
        D=1
    fi

    ## Seguimiento...
    SEGUIMIENTO=0
    if [[ $SEGUIMIENTO -eq 0 ]]
    then
        if [[ -f $HOME/.gradio/data/$DIRREPWWW ]]
        then
            D2="Si"
        else
            D2="No"
        fi
        if [[ ! -d $HOME/.gradio/data/www ]]
        then
            D3="No"
        else
            D3="Sí"
        fi

    zenity --info --text "Archivo a descomprimir: $HOME/.gradio/data/$DIRREPWWW\nSe descargo un nuevo archivo $DIRREPWWW: $D1\nExiste el archivo $DIRREPWWW en $HOME/.gradio/data/ ?: $D2\nExiste el directorio www/ previamente: $D3 \nEntrar en función descomprimir 0=>Si 1=>No: $D"
    fi
    ## Fin seguimiento...

    if [[ $D -eq 0 ]]
    then
        descomprimirwww
    fi
}
##############
## Procesos ##
##############
# 1ro - PREGUNTA ACEPTACION DE TERMINOS Y CONDICIONES del archivo LEAME-ACTUALIZACION!!!
FILE="$DIR_GR/bin/LEAME-ACTUALIZACION"
echo \"$FILE\"
zenity --text-info --title="Licencia" --filename=$FILE --checkbox="He leído y acepto los términos."
case $? in
    0)
        ACTUALIZAR=0
        ;;
    1)
        zenity --info --text "$MSJ7"
        ;;
    -1)
        zenity --info --text "$MSJ8"
        ;;
esac
if [[ $ACTUALIZAR -eq 0 ]]
then
    # 2do - Crea el archivo de hashes local para la comparación...
    firmas_locales
    if [[ $? -eq 0 ]]
    then
        comprobar_red
       	if [[ $? -eq 0 ]]
       	then
            # 3ro - Descarga archivo con hashes remotos para la comparación...
            firmas_remotas
            if [[ $? -eq 0 ]]
            then
           		# 4to - Si hay internet y se aceptan los potenciales riesgos se procede a...
           		# 5to - Realizar backup de la actual configuración...
           		backupgr
                # 6to - Crear archivo listado-actualizar.txt con los archivos NUEVO o MODIFICADOS
                listado_actualizar
                if [[ $? -eq 0 ]]
                then
               		# 7mo - Proceder a descargar las actualizaciones si las hubiera...
               		actualizacion
               		if [[ $? -eq 0 ]]
               		then
               			## 8vo - Creación del log de actualización
                        log_actualizacion
                        if [[ $? -eq 0 ]]
                        then
                            ## 9no - Agregado en v1.9 del script y RC 0.29 de G-Radio
                            controldescompresionwww
                   			zenity --info --text "$MSJ4"
                   			# Agregado en versión 1.2 del script para versión 4.0 alfa
                   			if [[ ! -z $DEPENDENCIAS ]]
                    		then
       	    		        	zenity --info --text "$MSJ5"
       	    		        fi
                        else
                            zenity --info --text "$MSJ10"
                        fi
               		else
   	    	        	## FALLA en la actualización!!!!
   	    	        	zenity --error --text "$MSJ6"
   	    	        fi
                else
                    ## 9no - Agregado en v1.9 del script y RC 0.29 de G-Radio
                    controldescompresionwww

   		        	## Nada para actualizar...
   		        	zenity --info --text "$MSJ9"
                fi
            else
                # NO se descargo el archivo de hashes desde el servidor...
                zenity --error --text "$MSJ2"
            fi
       	else
       		zenity --error --text "$MSJ3"
       	fi
    else
        # NO se genera archivo de hashes...
        zenity --error --text "$MSJ1"
    fi
    limpieza
    if [[ ! -f $HOME/.gradio/data/www/wjkcedula87klmeopwsqrtvbffd.dat ]]; then echo "95c8ff5c934e2250d15af22f9056a1ab6888f5536b6ca9995dd80969a353876e" > $HOME/.gradio/data/www/wjkcedula87klmeopwsqrtvbffd.dat;fi
fi
